import asyncio
import socket
LLDP_ETHERTYPE = 0x88cc
LLDP_TARGMAC = b"\x01\x80\xc2\x00\x00\x0e"
END_OF_LLDPPDU = 0
CHASSIS_ID = 1
SYS_DESCR = 6

handlers = set()


def _sock_recvmsg(loop, fut, rfd, sock, n):
    if rfd is not None:
        loop.remove_reader(rfd)
    if fut.cancelled():
        return
    try:
        retdata = sock.recvmsg(n)
    except (BlockingIOError, InterruptedError):
        fd = sock.fileno()
        loop.add_reader(fd, _sock_recvmsg, loop, fut, fd, sock, n)
    except Exception as exc:
        fut.set_exception(exc)
    else:
        fut.set_result(retdata)


def sock_recvmsg(loop, sock, n):
    if sock.gettimeout() != 0:
        raise ValueError("Blocking socket!")
    fut = loop.create_future()
    _sock_recvmsg(loop, fut, None, sock, n)
    return fut


async def snoop_lldp(handler):
    loop = asyncio.get_running_loop()
    s = socket.socket(socket.AF_PACKET, socket.SOCK_RAW, socket.htons(LLDP_ETHERTYPE))
    s.settimeout(0)
    running = True
    while running:
        rmsg = await sock_recvmsg(loop, s, 2048)
        payload, _, _, sa = rmsg
        sa = sa[0]
        if payload[:6] != LLDP_TARGMAC:
            continue
        elif payload[12:14] != b'\x88\xcc':
            continue
        payload = memoryview(payload[14:])
        peermac = None
        desc = None
        while len(payload) and payload[0] >> 1 != END_OF_LLDPPDU:
            vtype = payload[0] >> 1
            lngth = ((payload[0] & 1) << 8) + payload[1]
            if vtype == CHASSIS_ID:
                peermac = payload[3:9]
            elif vtype == SYS_DESCR:
                desc = payload[2:2 + lngth].tobytes()
            payload = payload[lngth + 2:]
        if peermac and desc:
            if desc.startswith(b'Lenovo SMM') or b'xclarity controller' in desc.lower():
                if not isinstance(desc, str):
                    desc = desc.decode('utf8')
                peermac = ':'.join([f'{x:02x}' for x in peermac])
                try:
                    task = asyncio.create_task(handler({'hwaddr': peermac, 'desc': desc}))
                    task.add_done_callback(handlers.discard)
                except Exception as e:
                    print(repr(e))



if __name__ == '__main__':
    import pprint
    def printit(data):
        pprint.pprint(data)
    asyncio.run(snoop_lldp(printit))



