import asyncio
import json
import shlex
import struct

def _sock_recvfrom(loop, fut, rfd, sock, n):
    if rfd is not None:
        loop.remove_reader(rfd)
    if fut.cancelled():
        return
    try:
        retdata = sock.recvfrom(n)
    except (BlockingIOError, InterruptedError):
        fd = sock.fileno()
        loop.add_reader(fd, _sock_recv_maybe_from, loop, fut, fd, sock, n)
    except Exception as exc:
        fut.set_exception(exc)
    else:
        fut.set_result(retdata)

def sock_recvmsg(sock, n):
    loop = asyncio.get_running_loop()
    if sock.gettimeout() != 0:
        raise ValueError("Blocking!")
    fut = loop.create_future()
    _sock_recvfrom(loop, fut, None, sock, n, True)
    return fut

async def run(cmd):
    cmd = shlex.split(cmd)
    cmd = await asyncio.subprocess.create_subprocess_exec(*cmd, stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.PIPE)
    return await cmd.communicate()


def mac2lla(mac):
    macbytes = bytearray([int(x, 16) for x in mac.split(':')])
    macbytes[0] = macbytes[0] ^ 2
    macbytes = macbytes[:3] + b'\xff\xfe' + macbytes[3:]
    pieces = struct.unpack('!HHHH', macbytes)
    return 'fe80::{:x}:{:x}:{:x}:{:x}'.format(*pieces)


async def get_interface(port):
    result, err = await run('ip -j vrf')
    result = json.loads(result)
    vrfs = {}
    for vrf in result:
        vid = vrf['table']
        vname = vrf['name']
        vrfs[vid] = vname
    result, err = await run(f'bridge -j vlan show dev {port}')
    vlif = json.loads(result)
    vlid = None
    for vl in vlif:
        for vlan in vl['vlans']:
            if 'PVID' in vlan.get('flags', []):
                vlid = vlan['vlan']
                break
    result, err = await run('ip -d -j link show')
    linkinf = json.loads(result)
    for link in linkinf:
        if link.get('linkinfo', {}).get('info_kind', '') != 'vlan':
            continue
        currid = link['linkinfo']['info_data']['id']
        if currid == vlid:
            vrfname = None
            if link.get('linkinfo', {}).get('info_slave_kind', None) == 'vrf':
                vrfid = link['linkinfo'].get('info_slave_data', {}).get('table', None)
                if vrfid in vrfs:
                    vrfname = vrfs[vrfid]
            return link['ifname'], link['ifindex'], vlid, vrfname
