import myutil
import base64
import asyncio
import apiclient
import json
from concurrent.futures import ProcessPoolExecutor

class NeighFetcher:
    def __init__(self, ipaddr, fingerprint):
        self.ipaddr = ipaddr
        bfingerprint = base64.b64decode(fingerprint)
        self.verifier = apiclient.CertVerifier(bfingerprint)
    
    def fetch(self):
        ipaddr = self.ipaddr
        verifier = self.verifier
        neighdata = apiclient.HTTPSClient(
            True, host=ipaddr, confluent=False, verifycallback=verifier.verify_cert).grab_url_with_status(
                '/scripts/neighdata.json')
        rsp = neighdata[1]
        if not isinstance(rsp, str):
            rsp = rsp.decode('utf8')
        neighdata = json.loads(rsp.replace('\n', ''))
        return neighdata

def is_smm(desc):
    return desc.lower().startswith('lenovo smm')

def is_xcc(desc):
    return desc.lower().startswith('lenovo xclarity controller')

async def get_chain(info, encnum, handle_info, prevmac=None):
    fingerprint = info['fingerprint']
    ipaddr = info['directaddress']
    scope = ipaddr.split('%')[1]
    nfetcher = NeighFetcher(ipaddr, fingerprint)
    info['id'] = 'smm1'
    info['enclosure'] = f'enclosure{encnum}'
    info['enclosure_num'] = encnum
    await handle_info(info)
    with ProcessPoolExecutor(1) as p:
        neighdata = await asyncio.get_event_loop().run_in_executor(p, nfetcher.fetch)
    for neighinfo in neighdata:
        desc = neighinfo.get('peerdesc', '')
        if is_smm(desc):
            peertype = 'lenovo-smm2'
            nextmac = neighinfo['peermac']
            if nextmac == prevmac:
                continue
            newinfo = fill_info(info, scope, neighinfo, peertype)
            await get_chain(newinfo, encnum + 1, handle_info, info['hwaddr'])
        elif is_xcc(desc):
            peertype = 'lenovo-xcc'
            newinfo = fill_info(info, scope, neighinfo, peertype)
            newinfo['id'] = 'xcc{0}'.format(neighinfo['port'] + 1)
            newinfo['bay'] = neighinfo['port'] + 1
            newinfo['enclosure'] = f'enclosure{encnum}'
            newinfo['enclosure_num'] = encnum
            await handle_info(newinfo)
     

def fill_info(info, scope, neighinfo, peertype):
    nextmac = neighinfo['peermac']
    nextaddr = myutil.mac2lla(nextmac)
    nextipaddr = f'{nextaddr}%{scope}'
    newinfo = {
                'directaddress': nextipaddr,
                'fingerprint': neighinfo.get('sha256', neighinfo.get('sha384', '')),
                'hwaddr': nextmac,
                'port': info['port'],
                'vrf': info['vrf'],
                'type': peertype,
                'desc': neighinfo['peerdesc'],
                'vlan': info['vlan'],
            }
    if len(newinfo['fingerprint']) == 68:
        newinfo['fingerprint'] = newinfo['fingerprint'][:64]
    return newinfo

