#!/usr/bin/python3
import asyncio
import base64
import codecs
import json
import glob
import neigh
import lldp
import ssdp
import base64
import forwarder
import hashlib
import os
import socket
from concurrent.futures import ProcessPoolExecutor
from aiohttp import web, web_urldispatcher, connector, ClientSession
import pam
import struct
import apiclient
import re
import myutil
import linkmon
import slp
import urllib.parse
import subprocess
import time
import smm


routes = web.RouteTableDef()
credcache = {}
RTM_GETNEIGH = 30
RTM_NEWNEIGH = 28
NLM_F_DUMP = 0x300
NLM_F_REQUEST = 1
NLMSG_DONE = 3

def msg_align(length):
    return (length + 3) & ~3



numregex = re.compile('([0-9]+)')

def fixuuid(baduuid):
    # SMM dumps it out in hex
    uuidprefix = (baduuid[:8], baduuid[8:12], baduuid[12:16])
    a = codecs.encode(struct.pack('<IHH', *[int(x, 16) for x in uuidprefix]),
        'hex')
    if not isinstance(a, str):
        a = a.decode('utf8')
    uuid = (a[:8], a[8:12], a[12:16], baduuid[16:20], baduuid[20:])
    return '-'.join(uuid).lower()

def naturalize_string(key):
    """Analyzes string in a human way to enable natural sort

    :param key: The node name to analyze
    :returns: A structure that can be consumed by 'sorted'
    """
    return [int(text) if text.isdigit() else text.lower()
            for text in re.split(numregex, key)]


def natural_sort(iterable):
    """Return a sort using natural sort if possible

    :param iterable:
    :return:
    """
    try:
        return sorted(iterable, key=naturalize_string)
    except TypeError:
        # The natural sort attempt failed, fallback to ascii sort
        return sorted(iterable)

# example of coping with vrf... have to get interface name
# from the scope index in sockaddr and set 25 option (SO_BINDTODEVICE)
# to work... attempts to bind the port directly did not result in an ability
# to actually transmit packets
#>>> a = socket.socket(socket.AF_INET6)
#>>> a.setsockopt(socket.SOL_SOCKET, 25, b'vlan1')
#>>> a.connect(('fe80::922e:16ff:fe06:e1c4%vlan1', 443))
#>>> a.send(b'hello\n')
#6
#>>> a.recv(16)
#b'HTTP/1.1 400 Bad'
#However, that's only if running under mgmt vrf, we'd run in default, in which case it's
#more strightforward

def get_interface_name(idx):
    for idxname in glob.glob('/sys/class/net/*/ifindex'):
        with open(idxname, 'r') as idxin:
            curridx = int(idxin.read())
        if curridx == idx:
            return idxname.split('/')[-2]

nlhdrsize = struct.calcsize('IHHII')
ndmsgsize = struct.calcsize('BiHBB')
# nlmsghdr struct
nlhdr = struct.pack('IHHII', nlhdrsize + ndmsgsize, RTM_GETNEIGH, NLM_F_DUMP|NLM_F_REQUEST, 0, 0)
ndmsg = struct.pack('BiHBB', 0, 0, 0, 0, 0)
neightable = {}
neighvintage = 0
async def refresh_neighbor_table():
    global neightable
    global neightime
    loop = asyncio.get_running_loop()
    s = socket.socket(socket.AF_NETLINK, socket.SOCK_RAW, socket.NETLINK_ROUTE)
    s.bind(0, 0)
    await loop.sock_sendall(s, nlhdr + ndmsg)
    neightable = {}
    neighvintage = os.times()[4]
    try:
        pending = True
        while pending:
            pdata = loop.sock_recv(s, 65536)
            v = memoryview(pdata)
            length = 1
            while length and len(v):
                length, typ = struct.unpack('IH')
                if typ == NLMSG_DONE:
                    pending = False
                    break
                if typ == RTM_NEWNEIGH:
                    ent = v[nlhdrsize:]
                    _, idx, state, flags, typ = struct.unpack('BiHBB', ent[:ndmsgsize])
                    if typ == 1:  # unicast
                        curraddr = None
                        currip = None
                        rta = ent[ndmsgsize:]
                        rtalen = 1
                        while rtalen and len(rta):
                            rtalen, rtatyp = struct.unpack('HH', rta[:4])
                            if rtatyp == 2:
                                curraddr = rta[4:rtalen].tobytes()
                                if len(curraddr) == 20:
                                    curraddr = curraddr[12:]
                            elif rtatyp == 1:
                                currip = rta[4:rtalen].tobytes()
                            rta = rta[msg_align(rtalen):]
                        if curraddr and currip:
                            neightable[currip] = curraddr
                v = v[msg_align(length):]
    finally:
        s.close()


validtokens = {}
async def password_bad(header):
    if header.startswith('Basic '):
        header = header.replace('Basic ', '')
        header = base64.b64decode(header).decode('utf8')
        user, passwd = header.split(':', 1)
        return await user_pass_bad(user, passwd)
    elif header.startswith('Bearer '):
        token = header.split(' ', 1)[1]
        if token in validtokens:
            validtokens[token]['expiry'] = time.time() + 120
            for clientip in validtokens[token].get('forwarders', []):
                cforwarder = validtokens[token]['forwarders'][clientip]
                cforwarder.allow_client(clientip)
            return False
        cxion = connector.UnixConnector(path='/run/affluent/httpapi')
        async with ClientSession(connector=cxion) as session:
            async with session.get('http://n/', headers={'Authorization': header}) as resp:
                await resp.text()
                if resp.status == 200:
                    validtokens[token] = {
                        'expiry': time.time() + 120,
                    }
                    return False
    return True


async def user_pass_bad(user, passwd):
    if user in credcache:
        crypted = hashlib.sha256(passwd.encode('utf8')).digest()
        if crypted == credcache[user]:
            return False
        del credcache[user]
    pid = os.fork()
    if not pid:
        auth = False
        try:
            os.setuid(0)
        except Exception:
            pass
        try:
            auth = pam.authenticate(user, passwd)
        finally:
            os._exit(0 if auth else 1)
    wpid = 0
    while wpid == 0:
        wpid, auth = os.waitpid(pid, os.WNOHANG)
        if wpid:
            auth = auth == 0
        await asyncio.sleep(0.1)
    if auth:
        passwd = passwd.encode('utf8')
        credcache[user] = hashlib.sha256(passwd).digest()
        return False
    return True

@web.middleware
async def fix_slashies(request, handler):
    if hasattr(request.match_info.route, 'status') and request.match_info.route.status == 404:
        if not request.path.endswith('/'):
            request = request.clone(rel_url=request.path + '/')
            minfo = await app.router.resolve(request)
            if not isinstance(minfo, web_urldispatcher.MatchInfoError):
                shortname = str(request.rel_url).split('/')[-2] + '/'
                raise web.HTTPMovedPermanently(shortname)
    return await handler(request)

@web.middleware
async def auth(request, handler):
    if request.path.endswith('/systems/get_notifications'):
        return await handler(request)
    if 'Authorization' not in request.headers or await password_bad(request.headers['Authorization']):
        return await refuse(request)
    return await handler(request)

async def refuse(request):
    return web.Response(status=401, body='Unauthorized', headers={'WWW-Authenticate': 'Basic'})

@routes.get('/')
async def top_get(request):
    return web.Response(text='["systems/", "discovery_subscribers/, "cert_authorities/"]')


@routes.post('/cert_authorities/{name}')
async def add_authority(request):
    certname = request.match_info['name']
    certdata = await request.text()
    fname = f'/etc/affluent/tls/authorities/{certname}.pem'
    os.makedirs('/etc/affluent/tls/authorities', exist_ok=True)
    with open(fname, 'w') as certout:
        certout.write(certdata)
    hasho = subprocess.check_output(['openssl', 'x509', '-in', fname, '-noout', '-hash'])
    hasho = hasho.decode('utf8').strip()
    hidx = 0
    hname = f'/etc/affluent/tls/authorities/{hasho}.{hidx}'
    while os.path.exists(hname):
        try:
            realhasho = os.readlink(hname)
            if realhasho == fname:
                break
        except OSError:
            pass
        hidx += 1
        hname = f'/etc/affluent/tls/authorities/{hasho}.{hidx}'
    if not os.path.exists(hname):
        os.symlink(fname, hname)
    return web.Response(text='{"status": 0}')

@routes.delete('/cert_authorities/{name}')
async def del_authority(request):
    certname = request.match_info['name']
    hidx = 0
    fname = f'/etc/affluent/tls/authorities/{certname}.pem'
    hasho = subprocess.check_output(['openssl', 'x509', '-in', fname, '-noout', '-hash'])
    hasho = hasho.decode('utf8').strip()
    hname = f'/etc/affluent/tls/authorities/{hasho}.{hidx}'
    while os.path.exists(hname):
        try:
            realhasho = os.readlink(hname)
            if realhasho == fname:
                os.remove(hname)
        except OSError:
            pass
        hidx += 1
        hname = f'/etc/affluent/tls/authorities/{hasho}.{hidx}'
    os.remove(fname)
    return web.Response(text='{"status": 0}')



@routes.get('/discovery_subscribers/')
async def fetch_subscribers(request):
    return web.Response(text=json.dumps([x for x in get_subscribers()]))

@routes.get('/discovery_subscribers/{name}')
async def fetch_one_subscriber(request):
    reqname = request.match_info['name']
    info = get_subscribers()[reqname]
    del info['authtoken']
    return web.Response(text=json.dumps(info))

@routes.delete('/discovery_subscribers/{name}')
async def delete_subscriber(request):
    reqname = request.match_info['name']
    currsubs = get_subscribers()
    if reqname in currsubs:
        del currsubs[reqname]
    save_subscribers(currsubs)
    return web.Response(text='{"status": 0}')

@routes.post('/discovery_subscribers/{name}')
async def add_subscriber(request):
    reqname = request.match_info['name']
    currsubs = get_subscribers()
    reqdata = await request.json()
    reqdata['name'] = reqname
    newpass, crypted = apiclient.gen_net_apikey()
    newpass = newpass.decode('utf8')
    crypted = crypted.decode('utf8')
    reqdata['authtoken'] = newpass
    currsubs[reqname] = reqdata
    save_subscribers(currsubs)
    return web.Response(text=f'{{"status": 0, "cryptkey": "{crypted}"}}')

@routes.get('/systems/')
async def top_get(request):
    return web.Response(text='["renotify", "by-port/"]')

@routes.post('/systems/renotify')
async def renotify_request(request):
    reqdata = await request.json()
    sub = reqdata['subscriber']
    await renotify(sub)
    return web.Response(text='{"status": "notifying"}')

@routes.get('/systems/all-data')
async def graball_request(request):
    return web.json_response(lldp_by_port)

@routes.get('/systems/by-port/')
async def get_ports(request):
    txt = json.dumps(['{}/'.format(x) for x in natural_sort(lldp_by_port)])
    return web.Response(text=txt)

@routes.get('/systems/by-port/{port}/')
async def get_port(request):
    port = request.match_info['port']
    if port not in lldp_by_port:
        return web.Response(status=404, body='Not Found')
    result = []
    for x in natural_sort(lldp_by_port[port]):
        if x == None:
            for y in natural_sort(lldp_by_port[port][x]):
                result.append(y)
        else:
            result.append(x)
    return web.Response(text=json.dumps(result))

@routes.get('/systems/by-port/{port}/{enclosure}/{endpoint}/')
@routes.get('/systems/by-port/{port}/{endpoint}/')
def get_opts(request):
    port = request.match_info['port']
    try:
        enclosure = request.match_info['enclosure']
    except KeyError:
        enclosure = None
    endpoint = request.match_info['endpoint']
    inf = lldp_by_port.get(port, {})
    if enclosure:
        inf = inf.get(enclosure, {})
        if endpoint in inf:
            return web.Response(text='["info", "webaccess"]')
    elif endpoint in inf:  # enclosure
        rsp = [f'{bmc}' for bmc in natural_sort(inf[endpoint])]
        return web.Response(text=json.dumps(rsp))
    if endpoint in inf.get(None, {}):
        return web.Response(text='["info", "webaccess"]')
    return web.Response(status=404, body='Not Found')


@routes.get('/systems/by-port/{port}/{enclosure}/{endpoint}/info')
@routes.get('/systems/by-port/{port}/{endpoint}/info')
async def get_individual_info(request):
    try:
        enclosure = request.match_info['enclosure']
    except KeyError:
        enclosure = None
    port = request.match_info['port']
    endpoint = request.match_info['endpoint']
    try:
        inf = lldp_by_port[port][enclosure][endpoint]
    except KeyError:
        return web.Response(status=404, body='Not Found')
    inf = jsonify(inf)
    return web.Response(text=inf)

def jsonify(inf):
    newinf = {}
    for key in inf:
        if isinstance(inf[key], bytes):
            newinf[key] = base64.b64encode(inf[key]).decode('utf8')
        elif key == 'directaddress':
            newinf['linklocal'] = inf[key].split('%')[0]
        else:
            newinf[key] = inf[key]
    inf = json.dumps(newinf, sort_keys=True)
    return inf

ws_notifiers = []
@routes.get('/systems/get_notifications')
async def notify_me(request):
    token = None
    ws = web.WebSocketResponse()
    await ws.prepare(request)
    authdone = False
    thews = None
    try:
        async for msg in ws:
            if not authdone:
                try:
                    authinfo = json.loads(msg.data)
                    authval = authinfo['authvalue']
                except Exception:
                    await ws.close()
                    return
                badauth = await password_bad(authval)
                if authval.startswith('Bearer '):
                    token = authval.split(' ', 1)[1]
                if badauth:
                    await ws.close()
                    return
                authdone = True
                thews = ws
                ws_notifiers.append(thews)
            try:
                validtokens[token]['expiry'] += 120
            except KeyError:
                pass
            for cli in validtokens.get(token, {}).get('forwarders', []):
                validtokens[token]['forwarders'][cli].allow_client(cli, token)
    finally:
        if thews:
            ws_notifiers.remove(thews)
        if token and token in validtokens:
            for clientip in validtokens[token].get('forwarders', []):
                validtokens[token]['forwarders'][clientip].close_client(clientip, token)


@routes.get('/systems/by-port/{port}/{enclosure}/{endpoint}/webaccess')
@routes.get('/systems/by-port/{port}/{endpoint}/webaccess')
async def get_webforwarder(request):
    targurl = await get_webforwarder_url(request)
    raise web.HTTPFound(targurl)


@routes.get('/systems/by-port/{port}/{enclosure}/{endpoint}/webaccessurl')
@routes.get('/systems/by-port/{port}/{endpoint}/webaccessurl')
async def get_webforwarder_no_redirect(request):
    targurl = await get_webforwarder_url(request)
    return web.Response(text=f'{{"targeturl": "{targurl}"}}')


async def get_webforwarder_url(request):
    port = request.match_info['port']
    endpoint = request.match_info['endpoint']
    try:
        enclosure = request.match_info['enclosure']
    except KeyError:
        enclosure = None
    try:
        inf = lldp_by_port[port][enclosure][endpoint]
    except KeyError:
        return web.Response(status=404, body='Not Found')
    clientip = request.headers['X-Real-Ip']
    targhost = request.headers['X-Forwarded-Host']
    tcpport, cforwarder = await forwarder.get_port(inf['directaddress'], clientip, inf['vrf'])
    if request.headers['Authorization'].startswith('Bearer '):
        token = request.headers['Authorization'].split(' ', 1)[1]
        if token in validtokens:
            validtokens[token]['expiry'] += 120
            if 'forwarders' in validtokens[token]:
                validtokens[token]['forwarders'][clientip] = cforwarder
            else:
                validtokens[token]['forwarders'] = {clientip: cforwarder}
    try:
        relpath = request.query['path']
        if relpath[0] != '/':
            relpath = '/' + relpath
    except KeyError:
        relpath = '/'
    return f'https://{targhost}:{tcpport}{relpath}'
    # pull in query to be able to specify path and put into the redirect
    


app = web.Application(middlewares=(auth, fix_slashies))
app.router.add_routes(routes)

info_by_port = {}
async def ssdp_detect(info):
    raise Exception('deprecated')
    port = await neigh.get_port(info[1][0])
    if port not in lldp_by_port:
        # drop for now, if lldp comes along, fire an active scan to try again
        # ssdp snoop is intended for chassis hosted servers
        return
    newinfo = {}
    newinfo['port'] = port
    saddr = info[1]
    newinfo['sockaddr'] = saddr
    newinfo['type'] = 'XCC'
    if len(saddr) == 4:
        targip = '{0}%{1}'.format(saddr[0], saddr[-1])
        verifier = apiclient.CertVerifier(lldp_by_port[port]['fingerprint'])
        try:
            devdesc = apiclient.HTTPSClient(True, host=targip, confluent=False, verifycallback=verifier.verify_cert).grab_url_with_status('/DeviceDescription.json')
            if devdesc[0] == 200:
                dd = json.loads(devdesc[1])
                if dd[0]['type'] == 'lenovo-xclarity-controller':
                    uuid = dd[0]['node-uuid'].strip().lower()
                    uuid = '-'.join([uuid[:8], uuid[8:12], uuid[12:16], uuid[16:20], uuid[20:]])
                    info_by_port[port] = {
                        'type': 'lenovo-xcc',
                        'directaddress': targip,
                        'fingerprint': base64.b64encode(lldp_by_port[port]['fingerprint']).decode('utf8'),
                        'name': dd[0]['hostname'].strip(),
                        'ipv4-addresses': dd[0]['ipv4-addresses'],
                        'uuid': uuid,
                        'serial': dd[0]['enclosure-serial-number'].strip(),
                        'model': dd[0]['enclosure-machinetype-model'].strip()
                    }
        except Exception as e:
            pass

def add_xcc_info(info):
    #TODO: this is currently blocking, attempts to use aiohttp fail because both it
    # and underlying asyncio fall over with link local addresses
    # either subprocess this out, deal with the hopefully minimal stutter, or
    # do something to try to rig an async http client that is LLA capable
    targip = info['directaddress']
    fprint = base64.b64decode(info['fingerprint'])
    verifier = apiclient.CertVerifier(fprint)
    try:
        devdesc = apiclient.HTTPSClient(True, host=targip, confluent=False, verifycallback=verifier.verify_cert, vrf=info['vrf']).grab_url_with_status('/DeviceDescription.json')
        if devdesc[0] == 200:
            dd = json.loads(devdesc[1])
            if dd[0]['type'] == 'lenovo-xclarity-controller':
                uuid = dd[0]['node-uuid'].strip().lower()
                uuid = '-'.join([uuid[:8], uuid[8:12], uuid[12:16], uuid[16:20], uuid[20:]])
                info['name'] =  dd[0]['hostname'].strip()
                info['type'] = 'lenovo-xcc'
                info['ipv4-addresses'] =  dd[0]['ipv4-addresses']
                info['uuid'] = uuid
                info['serialnumber'] = dd[0]['enclosure-serial-number'].strip()
                info['modelnumber'] = dd[0]['enclosure-machinetype-model'].strip()
    except Exception as e:
        pass

def add_smm_info(info):
    targip = info['directaddress']
    targaddr = socket.getaddrinfo(targip, 427, proto=socket.IPPROTO_TCP)[0][-1]
    newinfo = slp.query_target(targaddr)
    newinfo = newinfo['attributes']
    info['uuid'] = fixuuid(newinfo['uuid'][0])
    info['serialnumber'] = newinfo['enclosure-serial-number'][0]
    info['modelnumber'] = newinfo['enclosure-machinetype-model'][0]
    info['type'] = 'lenovo-smm2'

def handle_linkdown(iface):
    if iface in lldp_by_port:
        del lldp_by_port[iface]
        notify_webwatchers()

def save_subscribers(currsubs):
    with open('/etc/affluent/discovery_servers.json', 'w') as dislist:
        dislist.write(json.dumps(currsubs))

def get_subscribers():
    try:
        with open('/etc/affluent/discovery_servers.json', 'r') as dislist:
            return json.load(dislist)
    except (FileNotFoundError, json.decoder.JSONDecodeError):
        return {}

notifpool = None
async def notify_discovery(info, subinf=None):
    # here we hit another limitation of aiohttp
    # we need to set the mgmt vrf, but with aiohttp,
    # there isn't a nice hook to either pre-alloc
    # a workable sock, or else hook into it's sock
    # before it tries to use it.
    # use fork to perform each notification
    # in the background
    global notifpool
    mynotifsubs = set([])
    info = jsonify(info).encode('utf8')
    if subinf:
        subs = {'theone': subinf}
    else:
        subs = get_subscribers()
    for sub in subs:
        if notifpool is None:
            notifpool = ProcessPoolExecutor(12)
        subinf = subs[sub]
        nt = asyncio.get_event_loop().run_in_executor(notifpool, notify_listener, info, subinf)
        mynotifsubs.add(nt)
    for fut in mynotifsubs:
        await fut

def notify_listener(info, subinfo):
    try:
        url = subinfo['url']
        parsed = urllib.parse.urlparse(url)
        host = parsed.hostname
        myname = subinfo['authname']
        authtoken = subinfo['authtoken']
        print(repr(apiclient.HTTPSClient(True, host=host, vrf=b'mgmt', authname=myname, apikey=authtoken).grab_url_with_status(parsed.path, info)))
    except Exception as e:
        print(str(e))

lldp_by_port = {}
enrichers_by_type = {
    'lenovo-xcc': add_xcc_info,
    'lenovo-smm2': add_smm_info,
}
async def lldp_detect(info):
    port = await neigh.get_port_by_mac(info['hwaddr'])
    fprint = info['desc'].split(';')[-1]
    if fprint[-6:] == '\nAA==\n':
        fprint = fprint[:-6]
    info['fingerprint'] = fprint
    ginfo = await myutil.get_interface(port)
    if ginfo is None:
        raise Exception('Unable to get interface information, make sure at least a blank interface is defined on relevant vlan')
    mynic, myidx, vlid, vrfname = ginfo
    info['port'] = port
    info['vlan'] = vlid
    info['desc'] = ';'.join(info['desc'].split(';')[:-1])
    myip = myutil.mac2lla(info['hwaddr'])
    myip = f'{myip}%{myidx}'
    info['vrf'] = vrfname
    info['directaddress'] = myip
    if 'xclarity controller' in info['desc'].lower():
        info['type'] = 'lenovo-xcc'
        info['id'] = 'xcc1'
        await handle_info(info)
    elif 'Lenovo SMM' in info['desc']:
        info['type'] = 'lenovo-smm2'
        fprint = fprint.replace('S2=', '')
        info['fingerprint'] = fprint
        await smm.get_chain(info, 1, handle_info)

async def renotify(sub):
    target = get_subscribers()
    if sub in target:
        target = target[sub]
    else:
        raise Exception('Not apparently subscribed')
    for port in lldp_by_port:
        for enc in lldp_by_port[port]:
            for endpoint in lldp_by_port[port][enc]:
                info = lldp_by_port[port][enc][endpoint]
                nt = asyncio.create_task(notify_discovery(info, target))
                notify_tasks.add(nt)
                nt.add_done_callback(notif_done)


async def handle_info(info):
    fprint = info['fingerprint']
    port = info['port']
    enclosure = info.get('enclosure', None)
    if enclosure:
        info['path'] = '{}/{}/{}'.format(info['port'], enclosure, info['id'])
    else:
        info['path'] = '{}/{}'.format(info['port'], info['id'])
    endpoint = info['id']
    oldfprint = lldp_by_port.get(port, {}).get(enclosure, {}).get(endpoint, {}).get('fingerprint', None)
    if oldfprint == fprint:
        return
    enricher = enrichers_by_type[info['type']]
    enricher(info)
    if 'uuid' not in info:
        #enrichment failure, wait for more
        return
    if port not in lldp_by_port:
        lldp_by_port[port] = {}
    if enclosure not in lldp_by_port[port]:
        lldp_by_port[port][enclosure] = {}
    lldp_by_port[port][enclosure][endpoint] = info
    nt = asyncio.create_task(notify_discovery(info))
    notify_tasks.add(nt)
    nt.add_done_callback(notif_done)
    notify_webwatchers()

def notif_done(nt):
    global notifpool
    notify_tasks.discard(nt)
    if not notify_tasks:
        oldnp = notifpool
        notifpool = None
        if oldnp:
            oldnp.shutdown()

    
def notify_webwatchers():
    wnt = set([])
    for ws in ws_notifiers:
        nt = asyncio.create_task(ws.send_json({'detected': True}))
        wnt.add(nt)
        nt.add_done_callback(wnt.discard)

notify_tasks = set()
async def main():
    webtask = asyncio.ensure_future(web_main())
    #ssdptask = asyncio.create_task(ssdp.watch_ssdpv6(ssdp_detect))
    lldptask = asyncio.create_task(lldp.snoop_lldp(lldp_detect))
    linktask = asyncio.create_task(linkmon.notify_linkdown(handle_linkdown))
    for res in asyncio.as_completed([webtask, lldptask]):
        await res

async def web_main():
    runner = web.AppRunner(app)
    await runner.setup()
    site = web.UnixSite(runner, '/run/sysdisco/httpapi')
    await site.start()
    os.chmod('/run/sysdisco/httpapi', 0o666)
    while True:
        await asyncio.sleep(10)
        for token in list(validtokens):
            if validtokens[token]['expiry'] < time.time():
                del validtokens[token]
    #await site.cleanup()

if __name__ == '__main__':
    asyncio.run(main())
