// Copyright 2016 Lenovo
;(function() {
'use strict';
var logstatus;
var interactivecheck = false;

var authactive = false;

var codstr = {
    400 : "Bad Request",
    401 : "Authentication Failed",
    500 : "Internal Server Error",
    502 : "Service Unavailable",
    503 : "Service Unavailable",
    504 : "Service Unavailable"
}

async function show_issue(response) {
    if (response.status != 200) {
        return build_authui();
    }
    var txt = await response.text();
    var hd = await crypto.subtle.digest("SHA-256", (new TextEncoder()).encode(txt));
    var dgt = Array.from(new Uint8Array(hd)).map(x => x.toString(16).padStart(2), '0').join('');
    var olddgt = localStorage.getItem('IssueDigest')
    if (dgt == olddgt) {
        build_authui();
    } else {
        var warndlg = document.createElement('div');
        var blkout = document.getElementById("blackout");
	var warntxt = document.createElement('div');
        warntxt.textContent = txt;
	warndlg.style.background = "white";
	warndlg.appendChild(warntxt);
        var acknowledgeb = document.createElement('button');
        acknowledgeb.textContent = l("Acknowledge");
        acknowledgeb.addEventListener("click", function() {
            localStorage.setItem('IssueDigest', dgt);
            blkout.removeChild(warndlg);
            build_authui();
        });
        warndlg.appendChild(acknowledgeb);
        blkout.appendChild(warndlg);
    }
}

function clear_main() {
    closemain();
    CloseTerminals();
    var mn = document.getElementById('main');
    while (mn.firstChild) {
        mn.removeChild(mn.firstChild);
    }

    var errorDialogs = document.getElementsByClassName("modalbg");
    for (var i=0; i<errorDialogs.length; i++){
        var dialog = errorDialogs[i];
        var parent = dialog.parentElement;
        parent.removeChild(dialog);
    }
}
function set_transition() {
    var blkout = document.getElementById("blackout");
    var logform = document.getElementById("logindialog");
    blkout.style.transition="0.5s";
    //Wait for main to be obstructed before clearing it out
    setTimeout(clear_main, 500);
}


function create_secret_inputs(prompts) {
    parent = document.getElementById('secretinputs');
    prompts.forEach(prompt => {
        prompt = prompt.trim();
        for (let inputtag of parent.getElementsByTagName('input')) {
            if (inputtag.attributes.prompttext.value === prompt) {
                return;
            }
        }
        parent.appendChild(document.createTextNode(l(prompt)));
        var entryfield = document.createElement('input');
        entryfield.type = 'password';
        entryfield.setAttribute('prompttext', prompt);
        parent.appendChild(entryfield);
        parent.appendChild(document.createElement('br'));
    });
}

function create_authui() {
    var blkout = document.getElementById("blackout");
    if (blkout) {
        blkout.parentElement.removeChild(blkout);
    }
    blkout = document.createElement('div');
    blkout.id = 'blackout';
    document.body.appendChild(blkout);
    fetch('/confluent-public/web/issue').then(show_issue);
}
function build_authui() {
    var blkout = document.getElementById("blackout");
    var logdlg = document.createElement('div');
    logdlg.id = 'logindialog';
    var logform = document.createElement('form');
    logform.id = 'loginform';
    var userentry = document.createElement('input');
    userentry.id = 'username';
    userentry.type = 'text';
    userentry.setAttribute('spellcheck', 'false');
    userentry.value = localStorage.getItem('lastuser');
    var passentry = document.createElement('input');
    passentry.type = 'password';
    passentry.id = 'password';
    var logbutton = document.createElement('button');
    logbutton.type = 'submit';
    logbutton.id = 'loginbutton'
    logbutton.textContent = l('Login');
    var logo = document.createElement('img');
    logo.setAttribute('src', 'images/branding/current/logo.svg');
    logo.classList.add('logo');
    var sublogo = document.createElement('img');
    sublogo.setAttribute('src', 'images/branding/current/confluentlogo.svg');
    sublogo.classList.add('conflogo');
    logstatus = document.createElement('span');
    var csrf = document.createElement('input');
    csrf.setAttribute('type', 'hidden');
    csrf.setAttribute('name', 'csrf');
    csrf.setAttribute('value', Math.random().toString(36).substr(2,16));
    logform.appendChild(csrf);
    logform.appendChild(document.createTextNode(l('Username') + ':'));
    logform.appendChild(userentry);
    logform.appendChild(document.createElement('br'));
    var secretinputs = document.createElement('div');
    secretinputs.style.margin = "0";
    secretinputs.id = 'secretinputs';
    logform.appendChild(secretinputs);
    logform.appendChild(logstatus);
    logform.appendChild(logbutton);
    logdlg.appendChild(logo);
    logdlg.appendChild(sublogo);
    logdlg.appendChild(logform);
    blkout.appendChild(logdlg);
    document.body.appendChild(blkout);
    create_secret_inputs(['Password:']);
    document.getElementById("username").focus();
    logform.addEventListener("submit", check_auth);
}

var textClearer;
function show_auth(e, etxt, edata) {
    if (authactive) {
        if (interactivecheck) {
            if (textClearer) {
                clearTimeout(textClearer);
            }
            var logform = document.getElementById("logindialog");
            logform.style.animationName = 'shake';
    	    logstatus.style.color = 'darkred';
            var textContent = "Authentication Failed";
            if(codstr[e] != null){
                textContent = codstr[e];
            }
	    try {
            var prompts = JSON.parse(edata).prompts;
            create_secret_inputs(prompts);
	    } catch (error) {}
            logstatus.textContent = l("Error: " + textContent);
            textClearer = setTimeout(function() {
                logstatus.textContent = '';
            }, 5000);
        }
        return;
    }
    authactive = true;
    stopAsync();
    document.body.style.overflow = 'hidden';
    create_authui();
    var blkout = document.getElementById("blackout");
    blkout.style.opacity=1;
    blkout.style.pointerEvents="auto";
    setTimeout(set_transition, 0);
}

function clear_auth(sessinfo) {
    if (sessinfo['authtoken']) { setAuthToken(sessinfo['authtoken']); }
    if (sessinfo['sessionid']) { setSessionId(sessinfo['sessionid']); }
    localStorage.setItem('lastuser', sessinfo['username'])
    interactivecheck = false;
    authactive = false;
    document.body.style.overflow = 'visible';
    document.getElementById("banner").style.visibility = "visible";
    document.getElementById("navheader").style.visibility = "visible";
    startAsync(async_running.bind(sessinfo));
}
function async_running() {
    var sessinfo = this;
    window.noderange = defaultnoderange();
    main();
    document.getElementById("namedisplay").textContent = " " + sessinfo.username;
    var blkout = document.getElementById("blackout");
    if (blkout) {
        blkout.style.opacity = 0;
        blkout.style.pointerEvents = "none";
        blkout.parentElement.removeChild(blkout);
    }
}

function get_creds() {
    document.getElementById('allpassinfo')
}
function check_auth(e) {
    if (e.preventDefault) e.preventDefault();
    var logform = document.getElementById("logindialog");
    logform.style.animationName = null;
    var username = document.getElementById("username").value;
    var secrets = {};
    var nextinput = false;
    for (let inputtag of document.getElementById('secretinputs').getElementsByTagName('input')) {
        if (inputtag.value) {
            secrets[inputtag.attributes.prompttext.value] = inputtag.value;
        } else {
            nextinput = inputtag;
        }
    }
    if (nextinput) {
        // here we would check for authenticators for the user
        if (username) {
            authenticate_webauthn(username).then(m => m, e => nextinput.focus());
        } else {
            nextinput.focus()
        }
    } else {
        interactivecheck = true;
        logstatus.textContent = l('Checking Authentication..');
        getRequest("/confluent-api/sessions/current/info", clear_auth, show_auth, username, secrets, true);
    }
    return false;
}

function initauth() {
    getRequest("/confluent-api/sessions/current/info", clear_auth, show_auth, undefined, undefined, true);
}

function done_logout() {
    getRequest("/confluent-api/sessions/current/info", clear_auth, show_auth);
}
function logout() {
    stopAsync();
    getRequest("/confluent-api/sessions/current/logout", done_logout, done_logout);
}

async function authenticate_webauthn(name) {
	let authopts = await fetch_confluent('/confluent-api/sessions/current/webauthn/registered_credentials/' + name, {}, 'POST')
	authopts.challenge = b64tobuf(authopts.challenge)
	for (let i of authopts.allowCredentials) {
		i.id = b64tobuf(i.id)
	}
	authopts.userVerification = 'discouraged'
	let navcred = await navigator.credentials.get({publicKey: authopts});
	let credreq = {}
	credreq['raw_id'] = buftob64(navcred.rawId)
	navcred = navcred.response
	credreq['authenticator_data'] = buftob64(navcred['authenticatorData'])
	credreq['client_data_json'] = buftob64(navcred['clientDataJSON'])
	credreq['signature'] = buftob64(navcred.signature)
	credreq['user_handle'] = buftob64(navcred.userHandle)
	let authresult = await fetch_confluent('/confluent-api/sessions/current/webauthn/validate/' + name, credreq, 'POST')
	clear_auth(authresult)
}

window.logout = logout;
window.initauth = initauth;
window.show_auth = show_auth;
window.clear_main = clear_main;
window.authenticate_webauthn = authenticate_webauthn
})();
